import os, sys, pygame, random
from pygame.locals import *

class Textline(pygame.sprite.Sprite):
	def __init__(self, text, xpos, ypos, ttf, size, (r, g, b), aa=1, maxlen=None, extra=None):
		pygame.sprite.Sprite.__init__(self)
		self.color = (int(r), int(g), int(b))
		self.aa = int(aa)
		self.font = pygame.font.Font(ttf, int(size))
		self.text = text
		if extra:
			self.text = text + extra
		if maxlen != None:
			while self.font.size(text)[0] > int(maxlen):
				text = text[0:-1]
		if extra != None and text != ' ':
			text = text + extra
		self.image = self.font.render(text, self.aa, self.color)
		self.image = self.image.convert_alpha()
		width, height = self.image.get_size()
		self.rect = int(xpos), int(ypos), int(width), int(height)
		self.scrollcount = 0
		self.scrollinv = False
	
	def update(self, text, maxlen=None, extra=None):
		self.text = text
		if text == None:
			text = ' '
		if maxlen != None:
			if self.font.size(text)[0] > int(maxlen):
				while self.font.size(text)[0] > int(maxlen):
					text = text[0:-1]
		if extra != None and text != ' ':
			text = text + extra
		self.image = self.font.render(text, self.aa, self.color)
		self.image = self.image.convert_alpha()
		width, height = self.image.get_size()
		self.rect = self.rect[0], self.rect[1], int(width), int(height)
		self.scrollcount = 0
		self.scrollinv = False

	def scroll(self, maxlen, resx, speed=1):
		text = self.text
		if text == None:
			text = ' '
		if self.font.size(text)[0] > int(maxlen):
			image = self.font.render(text, self.aa, self.color)
			width, height = image.get_size()
			orgwidth = width
			choprect = 0, height, self.scrollcount, height
			self.image = pygame.transform.chop(image, choprect)
			choprect = maxlen, height, resx, height
			self.image = pygame.transform.chop(self.image, choprect)
			self.image = self.image.convert_alpha()
			width, height = self.image.get_size()
			self.rect = self.rect[0], self.rect[1], int(width), int(height)
			if orgwidth < maxlen + self.scrollcount:
				self.scrollinv = True
			elif self.scrollcount == 0:
				self.scrollinv = False
			if self.scrollinv: self.scrollcount -= speed
			else: self.scrollcount += speed
			

	def scroll2(self, maxlen, extra=None):
		text = self.text
		if text == None:
			text = ' '
		if self.font.size(text)[0] > int(maxlen):
			while self.font.size(text)[0] > int(maxlen):
				text = text[0:-1]
			textlen = len(text)
			text = self.text
			div = float(len(self.text)) / float(textlen)
			div = roundup(div)
			text = text[self.scrollcount*textlen:self.scrollcount*textlen+textlen]
			self.scrollcount += 1
			if self.scrollcount == div:
				self.scrollcount = 0
			if extra != None and text != ' ':
				text = text + extra
			self.image = self.font.render(text, self.aa, self.color)
			self.image = self.image.convert_alpha()
			width, height = self.image.get_size()
			self.rect = self.rect[0], self.rect[1], int(width), int(height)


def roundup(number):
	number = str(number).split('.')
	if number[1] != '0':
		return int(number[0]) + 1
	else:
		return int(number[0])

class Image(pygame.sprite.Sprite):
	def __init__(self, file, xpos=0, ypos=0, alpha=None, w=None, h=None):
		pygame.sprite.Sprite.__init__(self)
		self.image = load_image(file, alpha, w, h)
		width, height = self.image.get_size()
		self.rect = int(xpos), int(ypos), int(width), int(height)
	
	def update(self,x,y):
		self.rect = x,y,self.rect[2],self.rect[3]
		

class Quad(pygame.sprite.Sprite):
	def __init__(self, color, w, h, x=0, y=0):
		pygame.sprite.Sprite.__init__(self)
		self.image = pygame.Surface([int(w), int(h)])
		self.image.fill(color)
		self.rect = x, y, w, h
	
	def update(self, color):
		self.image.fill(color)
		
class Screensaver:
	def __init__(self, w,h, img, effectlist):
		self.img = pygame.sprite.RenderUpdates()
		image = Image(img, 0, 0)
		self.columns = round(w/image.rect[2]) + 1
		self.rows = round(h/image.rect[3]) + 1
		i = 0
		while i < self.rows:
			ii = 0
			while ii < self.columns:
				self.img.add(Image(img, image.rect[2]*ii, image.rect[3]*i))
				ii += 1
			i += 1
	
	def upd(self):
		for quad in self.quads.sprites():
			rand = random.randint(1, 10)
			updown = random.randint(0, 1)
			col = quad.image.get_at((0,0))
			nucol = col[0]
			if updown==0:
				if col[0]+rand<255:
					nucol = col[0]+rand
			elif updown==1:
				if col[0]-rand>0:
					nucol = col[0]-rand
			quad.update((nucol, nucol, nucol))
		
def load_image(fullname, alpha=None, w=None, h=None):
	#print 'trying to load image: ', fullname
	try:
		image = pygame.image.load(fullname)
	except:
		image = pygame.Surface([100, 100])
		image.fill((0,0,0))
		alpha = 0
	if w != None:
		image = pygame.transform.scale(image, (int(w), int(h)))
	if alpha != None:
		image.set_alpha(int(alpha))
	if image.get_alpha() is None:
		image = image.convert()
	else:
		image = image.convert_alpha()
	return image

def imgcalc(path, x, y):
	img = load_image(path)
	w, h = img.get_size()
	if w > x or h > y:
		if w >= h:
			d = float(w)/float(x)
			h = round(h/d)
			w = x
		elif w < h:
			d = float(h)/float(y)
			w = round(w/d)
			h = y
	posx = (x/2) - (w/2)
	posy = (y/2) - (h/2)
	return posx, posy, w, h
