﻿#!/usr/bin/env python
# -*- coding: <utf-8> -*-

try:
	# trying to import 'psyco' which precompiles
	# the program in order to make it faster
	# [http://psyco.sourceforge.net/]
	import psyco
	psyco.full()
except ImportError:
	pass
import os, sys, pygame, time
from pygame.locals import *
sys.path.append(os.path.join("data"))
from epm_gfx import *
from epm_core import *
from epm_xml import *


class epMEDIA:
	def __init__(self):
		# using ConfigObj as configfile-handler
		# [http://www.voidspace.org.uk/python/configobj.html]
		#self.conf = ConfigObj('epmedia.conf')
		self.cf = XmlReader()
		self.cf.openFile('epconfig.xml')
		if self.cf.val['config.sound'] == 'pysonic':
			import pySonic
			w = pySonic.World()
		self.themeloader()
		self.displayinit()
		#pySonic init
		# creating the playlist object out of epm_core
		self.playlist = Playlist(self.cf.val['config.sound'])
		self.font = os.path.join('themes', self.cf.val['config.theme'], 'font.ttf')
		# layers is just a list with spritelists in itself, this was neccesary to
		# get shure that things get rendered in the right order. Every layer can contain
		# multiple sprites
		self.layers = []
		self.layers.append(pygame.sprite.RenderUpdates())
		self.layers.append(pygame.sprite.RenderUpdates())
		self.layers.append(pygame.sprite.RenderUpdates())
		self.layers.append(pygame.sprite.RenderUpdates())
		self.layers.append(pygame.sprite.RenderUpdates())
		self.layers.append(pygame.sprite.RenderUpdates())
		self.layers.append(pygame.sprite.RenderUpdates())
		self.layers.append(pygame.sprite.RenderUpdates())
		self.layers.append(pygame.sprite.RenderUpdates())
		self.layers.append(pygame.sprite.RenderUpdates())
		self.list = List()
		self.drawobjectnum()
		self.mainmenu()
		self.playlist.load()
		self.drawdate()
		self.drawtrack()
		self.drawcover()
		self.drawplaytime()
		self.pictrigger = False
		clock = pygame.time.Clock()
		counter = []
		print 'Welcome to epMEDIA !'
		#effectimg = [self.themeimg('effect_on.png')]
		#scr = Screensaver(self.x, self.y, self.themeimg('effect.png'), effectimg)
		#self.layers[9] = scr.img
		while 1:
			# date and time is updated every frame
			self.datetime.update(time.strftime(self.cf.val['config.dateformat'],time.localtime()))
			self.playtime.update(self.playlist.time())
			# rectlist will contain the rect's that have to be updated
			rectlist = []
			# the rect's of the layers are added to the rectlist
			for layer in self.layers:
				rectlist.extend(layer.draw(self.screen))
			# with display.update only nedded parts of the display are redrawn
			pygame.display.update(rectlist)
			clock.tick(self.cf.val['config.fps'])
			#scr.upd()
			self.track.scroll(self.th.val['theme.curplay.length'], self.x)
			if self.WHERE == 'filebrowser':
				self.drawnlist[self.list.marker].scroll(self.th.val['theme.filebrowser.maxlength'], self.x, 2)
			#clearing the layers
			for layer in self.layers:
				layer.clear(self.screen, self.bg)
			# event (user input) handling
			for event in pygame.event.get((KEYDOWN, KEYUP, QUIT)):
				if event.type == QUIT: self.quit()
				if event.type == KEYDOWN:
					self.keyhandler(event.key)
				if event.type == KEYUP:
					# on keyup the preview-picture is shown again
					# as they where invisible while scrolling for
					# peformance reason
					self.layers[2].empty()
					self.updatepreview()
					if self.pictrigger:
						self.showpicture()
					# if no track is playing, but it should be (determined by playlist.playing)
					# play the next track , this happens every time a track ends
			if self.playlist.isplaying() == False and self.playlist.out.playing:
				print 'trying to open: ', self.playlist.track(False,True)
				self.playlist.next()
				self.trackupdate()
				self.drawcover()
				self.dolist()
	
	def themeloader(self):
		self.th = XmlReader()
		self.th.openFile(os.path.join('themes', self.cf.val['config.theme'], "theme.xml"))
		if self.cf.val['config.style'] != 'standard' and os.path.isfile(os.path.join('themes', self.cf.val['config.theme'], self.cf.val['config.style'], "theme.xml")):
			style = XmlReader()
			style.openFile(os.path.join('themes', self.cf.val['config.theme'], self.cf.val['config.style'], "theme.xml"))
			for key in style.val.keys():
				self.th.val[key] = style.val[key]
		self.x = self.th.val['theme.resX']
		self.y = self.th.val['theme.resY']

	def themeimg(self, img):
		if os.path.isfile(os.path.join('themes', self.cf.val['config.theme'], self.cf.val['config.style'], img)):
			return os.path.join('themes', self.cf.val['config.theme'], self.cf.val['config.style'], img)
		else:
			return os.path.join('themes', self.cf.val['config.theme'], img)

	def keyhandler(self, key):
		if key == K_ESCAPE: self.k_escape()
		if key == K_LCTRL: self.k_enter()
		if key == K_UP: self.k_up()
		if key == K_DOWN: self.k_down()
		if key == K_BACKSPACE: self.k_pgup()
		if key == K_TAB: self.k_pgdn()
		if key == K_LSHIFT: self.k_goto()
		if key == K_LALT: self.k_back()
		if key == K_LEFT: self.k_rew()
		if key == K_RIGHT: self.k_skip()
		if key == K_SPACE: self.k_sidemenu()
		if key == K_RETURN: self.mainmenu()

	def k_playlist():
		self.showplaylist()

	def k_sidemenu(self):
		if self.sec_program == 'PLAY_MUSIC':
			self.switchsidemenu()

	def k_addall(self):
		# add's all audio files in the music-section to
		# the playlist (may take a while), works only
		# if you are in the music section
		if self.sec_program == "PLAY_MUSIC":
			print 'trying to add all files of your music section'
			self.playlist.super_add(self.list.main_path, self.sec_ftypes)
			self.trackupdate()
			self.drawcover()
			self.dolist()

	def k_shuffle(self):
		# shuffles the playlist
		self.playlist.shuffle()
		self.trackupdate()
		self.drawcover()
		self.dolist()

	def k_playpause(self):
		# pauses playback
		self.playlist.playpause()

	def k_stop(self):
		# stops playback, clears playlist
		self.playlist.stop()
		self.trackupdate()
		self.drawcover()
		self.dolist()

	def k_skip(self):
		# play the next song in the playlist
		self.playlist.next()
		self.trackupdate()
		self.drawcover()
		self.dolist()

	def k_rew(self):
		# play the last song in the playlist
		self.playlist.prev()
		self.trackupdate()
		self.drawcover()
		self.dolist()

	def k_back(self):
		# go's out of that folder, or to the mainmenu
		self.go_back()
		self.objectnumupdate()

	def k_goto(self):
		# go's into the item, if it's a folder, if not, it does nothing	
		if self.WHERE == 'filebrowser' or self.WHERE == 'main':
			self.go_to()
			self.objectnumupdate()
			self.updatebar()
			self.layers[2].empty()

	def k_pgdn(self):
		# go's [length of visible list](defined in the skins) item down
		if self.WHERE == 'main':
			self.list.pgdown(self.th.val['theme.mainmenu.maxlines'])
		else:
			self.list.pgdown(self.th.val['theme.filebrowser.maxlines'])
			self.objectnumupdate()
		self.updatebar()
		self.dolist()
		self.layers[2].empty()

	def k_pgup(self):
		# go's [length of visible list](defined in the skins) item up
		if self.WHERE == 'main':
			self.list.pgup(self.th.val['theme.mainmenu.maxlines'])
		else:
			self.list.pgup(self.th.val['theme.filebrowser.maxlines'])
			self.objectnumupdate()
		self.updatebar()
		self.dolist()
		self.layers[2].empty()

	def k_down(self):
		# go's one item down
		pos = self.list.start
		if self.WHERE == 'main':
			self.list.next(self.th.val['theme.mainmenu.maxlines'])
		else:
			self.list.next(self.th.val['theme.filebrowser.maxlines'])
			self.objectnumupdate()
		self.updatebar()
		if pos != self.list.start:
			self.dolist()
		self.layers[2].empty()

	def k_up(self):
		# go's one item up
		pos = self.list.start
		if self.WHERE == 'main':
			self.list.prev(self.th.val['theme.mainmenu.maxlines'])
		else:
			self.list.prev(self.th.val['theme.filebrowser.maxlines'])
			self.objectnumupdate()
		self.updatebar()
		if pos != self.list.start:
			self.dolist()
		self.layers[2].empty()

	def k_escape(self):
		# go's out of that folder, or to the mainmenu, or quits
		if self.pictrigger:
			self.layers[8].empty()
			self.layers[9].empty()
			self.pictrigger = False
		elif self.WHERE == 'filebrowser' or self.WHERE == 'playlist':
			self.go_back()
			self.objectnumupdate()
		elif self.WHERE == 'main':
			self.quit()
		elif self.WHERE == 'sidemenu':
			self.switchsidemenu()

	def k_enter(self):
		# pictrigger says that we are in imageviewer mode
		if self.pictrigger:
			self.layers[8].empty()
			self.layers[9].empty()
			self.pictrigger = False
		elif self.WHERE == 'sidemenu':
			self.sidemenuaction()
		else:
			# file_handle will do what's neccesary after that
			# we are going into that folder (if it is)
			# and redraw the bar and the list
			self.file_handle()
			self.go_to()
			self.objectnumupdate()
			self.updatebar()
			self.dolist()

	def displayinit(self):
		print 'pygame engine init'
		pygame.init()
		print 'setting resolution to ', self.x, 'x', self.y
		if self.cf.val['config.fullscreen'] == 1:
			self.screen = pygame.display.set_mode((self.x, self.y), FULLSCREEN)
		else:
			self.screen = pygame.display.set_mode((self.x, self.y))
		pygame.display.set_caption('epMEDIA')
		pygame.display.set_icon(pygame.image.load(os.path.join('data', 'icon128x128.png'))) 
		pygame.mouse.set_visible(False)
		pygame.event.set_blocked(MOUSEMOTION)
		pygame.key.set_repeat(200, 20)

	def sidemenuaction(self):
		if self.list.marked_index() == 0:
			if os.path.isdir(self.actionitem):
				self.playlist.add_dir(self.actionitem, self.sec_ftypes, 0, True)
			elif os.path.isfile(self.actionitem):
				self.playlist.add(self.actionitem)
			self.switchsidemenu()
		elif self.list.marked_index() == 1:
			self.switchsidemenu()
			self.showplaylist()
		elif self.list.marked_index() == 2:
			self.switchsidemenu()
			self.k_addall()
		elif self.list.marked_index() == 3:
			self.k_shuffle()
			self.switchsidemenu()
		elif self.list.marked_index() == 4:
			if self.playlist.playpause() == False:
				self.switchsidemenu()
		elif self.list.marked_index() == 5:
			self.k_stop()
			self.switchsidemenu()
			if self.WHERE == 'playlist':
				self.go_back()

	def sidemenu(self, x, y, space, labels, lspan, hspan, fontsize, (r,g,b), xbar, ybar):
		self.list.store()
		self.side_oldbar = self.bar
		self.list.destroy()
		self.list.data = labels
		#self.layers[0].empty()
		yy = y
		for label in labels:
			self.layers[5].add(Image(self.themeimg('sidebutton.png'), x, y))
			self.layers[6].add(Textline(label, x + lspan, y + hspan, self.font, fontsize, (r, g, b),
			self.cf.val['config.fontaa']))
			y += space
		self.bar = Image(self.themeimg('sidebar.png'), xbar, ybar)
		self.layers[7].add((self.bar))

	def switchsidemenu(self, init=False):
		if self.WHERE == 'filebrowser' or self.WHERE == 'playlist':
			self.actionitem = self.list.marked_path()
			lab = ['add to playlist', 'view playlist', 'add all', 'shuffle', 'play/pause', 'stop']
			r,g,b = self.th.val['theme.sidemenu.color']
			self.layers[5].empty()
			self.layers[6].empty()
			self.sidemenu(self.th.val['theme.sidemenu.xpos'], self.th.val['theme.sidemenu.ypos'],
			self.th.val['theme.sidemenu.space'], lab, self.th.val['theme.sidemenu.hspan'],
			self.th.val['theme.sidemenu.vspan'], self.th.val['theme.sidemenu.fontsize'],
			(r,g,b), self.th.val['theme.sidemenu.xbar'], self.th.val['theme.sidemenu.ybar'])
			self.WHERE1 = self.WHERE
			self.WHERE = 'sidemenu'
			if init:
				self.switchsidemenu()
		elif self.WHERE == 'sidemenu' :
			self.WHERE = self.WHERE1
			self.list.destroy()
			self.list.restore()
			self.layers[7].empty()
			self.bar = self.side_oldbar
			self.layers[0].add(self.bar)
			self.dolist()
		else:
			pass

	def mainmenu(self):
		self.WHERE = 'main'
		self.posinlist = {}
		self.layers[3].remove(self.objectnum)
		self.list.destroy()
		x = 0
		while 1:
			try:
				self.list.data.append(self.cf.val['config.sections.section' + str(x) + '.long'])
			except(KeyError):
				break
			x += 1
		self.bg = load_image(self.themeimg('mainbg.png'))
		self.screen.blit(self.bg,(0,0))
		self.bar = Image(self.themeimg('mainbar.png'),
					self.th.val['theme.mainmenu.bar.xpos'], self.th.val['theme.mainmenu.bar.ypos'])
		pygame.display.update()
		self.layers[0].add((self.bar))
		self.dolist()
		self.updatepreview()
		self.layers[5].empty()
		self.layers[6].empty()
	
	def showplaylist(self):
		if self.playlist.tracks and self.WHERE == 'filebrowser':
			self.WHERE = 'playlist'
			self.list.store2()
			self.list.destroy()
			self.list.generate2(self.playlist.tracks)
			self.dolist()
			self.updatebar()

	def drawlist(self, x, y, space, maxlines, maxlen, fontsize, (rr,gg,bb)):
		self.drawnlist = []
		self.layers[1].empty()
		i = 0
		for item in self.list.data[self.list.start:]:
			r,g,b = rr, bb, gg
			xtra = None
			if self.WHERE == 'filebrowser' or self.WHERE == 'playlist':
				txt = item.name
				if item.type == 'file':
					if self.playlist.tracks:
						if item.name == self.playlist.track(True):
							r,g,b = self.th.val['theme.filebrowser.hilightRGB']
				elif item.type == 'dir':
					if self.playlist.tracks:
						pa, gr = os.path.split(self.playlist.track())
						if os.path.join(self.list.path, item.path) == pa:
							r,g,b = self.th.val['theme.filebrowser.hilightRGB']
					txt = '[' + txt
					xtra = ']'
			elif self.WHERE == 'main':
				txt = item
			self.drawnlist.append(Textline(txt, x, y, self.font, fontsize, (r, g, b),
						self.cf.val['config.fontaa'], maxlen, xtra))
			self.layers[1].add(self.drawnlist[i])
			y += space
			i += 1
			if i == maxlines: break

	def dolist(self):
		if self.WHERE == 'main':
			self.drawlist(self.th.val['theme.mainmenu.xpos'], self.th.val['theme.mainmenu.ypos'],
			self.th.val['theme.mainmenu.linespace'], self.th.val['theme.mainmenu.maxlines'],
			self.th.val['theme.mainmenu.maxlength'], self.th.val['theme.mainmenu.fontsize'],
			(self.th.val['theme.mainmenu.color']))
		elif self.WHERE == 'filebrowser':
			self.drawlist(self.th.val['theme.filebrowser.xpos'], self.th.val['theme.filebrowser.ypos'],
			self.th.val['theme.filebrowser.linespace'], self.th.val['theme.filebrowser.maxlines'],
			self.th.val['theme.filebrowser.maxlength'], self.th.val['theme.filebrowser.fontsize'],
			(self.th.val['theme.filebrowser.color']))
		elif self.WHERE == 'playlist':
			self.drawlist(self.th.val['theme.filebrowser.xpos'], self.th.val['theme.filebrowser.ypos'],
			self.th.val['theme.filebrowser.linespace'], self.th.val['theme.filebrowser.maxlines'],
			self.th.val['theme.filebrowser.maxlength'], self.th.val['theme.filebrowser.fontsize'],
			(self.th.val['theme.filebrowser.color']))
		else:
			pass

	def ispic(self, name):
		(shortname, ext) = os.path.splitext(name)
		if ext.lower() == ".jpg" or ext.lower() == ".jpeg" or ext.lower() == ".bmp" or ext.lower() == ".gif" or ext.lower() == ".png":
			return True
		else:
			return False

	def drawpreview(self, x, y, size=None, alpha=None):
		pic = None
		resize = 1
		if self.WHERE == 'main':
			i = 0
			while 1:
				if i == self.list.marker + self.list.start:
					try:
						short = self.cf.val['config.sections.section' + str(i) + '.short']
					except(KeyError):
						break
					break
				i += 1
			if os.path.isfile(self.themeimg(short + '_main_ico.png')):
				pic = self.themeimg(short + '_main_ico.png')
				resize = 0
		#it's a dir, so search for an image!
		elif self.list.isdir():
			inp = os.listdir(self.list.marked_path())
			for line in inp:
				if self.ispic(line):
					#a pic was found, no need to find more, break
					pic = os.path.join(self.list.marked_path(), line)
					break
		#it's a file
		elif self.list.isfile():
			#even better, it's an imagefile
			if self.ispic(self.list.marked_path()):
				pic = self.list.marked_path()
		#not a file, not a dir: mainmenu! if there are images for the sections, show them
		#so if we got an image, if needed it will be resized and then blitted
		if pic:
			previmg = Image(pic, x, y, alpha, size, size)
			self.layers[2].add((previmg))

	def updatepreview(self):
		if self.WHERE == 'filebrowser':
			self.drawpreview(self.th.val['theme.filebrowser.preview.xpos'], self.th.val['theme.filebrowser.preview.ypos'],
			self.th.val['theme.filebrowser.preview.size'], self.th.val['theme.filebrowser.preview.alpha'])
		elif self.WHERE == 'main':
			self.drawpreview(self.th.val['theme.mainmenu.preview.xpos'], self.th.val['theme.mainmenu.preview.ypos'])
		else:
			pass

	def drawdate(self):
		txt = time.strftime(self.cf.val['config.dateformat'],time.localtime())
		r,g,b = self.th.val['theme.time.color']
		self.datetime = Textline(txt, self.th.val['theme.time.xpos'], self.th.val['theme.time.ypos'], self.font,
		self.th.val['theme.time.fontsize'], (r, g, b), self.cf.val['config.fontaa'])
		self.layers[3].add(self.datetime)

	def drawtrack(self):
		r,g,b = self.th.val['theme.curplay.color']
		x = self.th.val['theme.curplay.xpos']
		y = self.th.val['theme.curplay.ypos']
		fsize = self.th.val['theme.curplay.fontsize']
		txt = ' '
		if self.playlist.track() != None:
			txt = self.playlist.track(True)
		self.track = Textline(txt, x, y, self.font, fsize, (r,g,b),
		self.cf.val['config.fontaa'], self.th.val['theme.curplay.length'])
		#self.nexttrack = Textline(txt2, x , y + fsize + 5, self.font, fsize - 2, (r,g,b), self.conf['smooth_text'])
		self.layers[3].add(self.track)
		#self.layers[3].add(self.nexttrack)
	
	def drawcover(self):
		self.layers[4].empty()
		if self.playlist.track() != None:
			path, junk = os.path.split(self.playlist.track())
			pic = None
			for line in os.listdir(path):
				if self.ispic(line):
					pic = os.path.join(path, line)
					self.coverpic = pic
					break
			if pic != None:
				self.cover = Image(pic, self.th.val['theme.cover.xpos'],
				self.th.val['theme.cover.ypos'], self.th.val['theme.cover.alpha'],
				self.th.val['theme.cover.size'], self.th.val['theme.cover.size'])
				self.layers[4].add(self.cover)

	def drawplaytime(self):
		txt = self.playlist.time()
		r,g,b = self.th.val['theme.playtime.color']
		self.playtime = Textline(txt, self.th.val['theme.playtime.xpos'], self.th.val['theme.playtime.ypos'],
		self.font, self.th.val['theme.playtime.fontsize'], (r, g, b), self.cf.val['config.fontaa'])
		self.layers[3].add(self.playtime)
		
	def drawobjectnum(self):
		txt = '[' + str(self.list.marked_index()+1)+'/'+str(len(self.list.data))+']'
		r,g,b = self.th.val['theme.filebrowser.objectnum.color']
		self.objectnum = Textline(txt, self.th.val['theme.filebrowser.objectnum.xpos'], self.th.val['theme.filebrowser.objectnum.ypos'],
		self.font, self.th.val['theme.filebrowser.objectnum.fontsize'], (r, g, b), self.cf.val['config.fontaa'])
	
	def objectnumupdate(self):
		if self.WHERE != 'sidemenu':
			self.objectnum.update('[' + str(self.list.marked_index()+1)+'/'+str(len(self.list.data))+']')

	def quit(self):
		print 'Goodbye!'
		self.playlist.save()
		pygame.quit()
		raise SystemExit

	# the filehandler
	def file_handle(self):
		# let's see what we have here...
		if self.WHERE == 'playlist':
			self.playlist.pos = self.list.start + self.list.marker
			self.playlist.play()
			self.drawcover()
			self.trackupdate()
		elif self.WHERE == 'main':
			pass
		elif self.list.isfile():
			# it's a file, what happens next depends on it's extension and what
			# our epmedia.conf handles
			# if whe have set 'PLAY_MUSIC' or 'SHOW_PICTURE' the file is handled by epMEDIA
			if self.sec_program == "PLAY_MUSIC":
				if self.list.marked_path()[-3] == 'm3u':
					# adds the filenames of the file to our playlist
					self.playlist.load(self.list.marked_path())
				else:
					# adds all files of the directory where that audiofile is in
					# to the playlist, plays marked file first
					p, j = os.path.split(self.list.marked_path())
					self.playlist.add_dir(p, self.sec_ftypes, self.list.marked_path())
				self.trackupdate()
				self.drawcover()
			elif self.sec_program == "SHOW_PICTURE":
				self.showpicture()
			else:
				# pygame stops. This was mainly necessary for ZSNES, which doesn't start with running
				# pygame display, also it saves a lot cpu-time for the launched application
				self.playlist.pause()
				# quitting the display
				pygame.display.quit()
				# launching the application with filename parameters
				os.system(self.sec_program + ' "' + self.list.marked_path() + '"')
				# bringing up the display again
				self.displayinit()
				self.screen.blit(self.bg,(0,0))
				pygame.display.update()
				self.playlist.playpause()
		# if it's a dir we add (if there are) the files in it to the playlist
		elif self.list.isdir():
			if self.sec_program == "PLAY_MUSIC":
				self.playlist.add_dir(self.list.marked_path(), self.sec_ftypes)
				self.trackupdate()
				self.drawcover()
			elif self.sec_program == "SHOW_PICTURE":
				self.showpicture()

	def trackupdate(self):
		self.track.update(self.playlist.track(True), self.th.val['theme.curplay.length'])

	def showpicture(self):
		self.layers[8].empty()
		self.layers[9].empty()
		self.pictrigger = False
		if self.list.isfile():
			blank = Quad((255,255,255), self.x, self.y)
			self.layers[8].add((blank))
			x,y,w,h = imgcalc(self.list.marked_path(), self.x, self.y)
			picture = Image(self.list.marked_path(),x,y,None,w,h)
			self.layers[9].add((picture))
			self.pictrigger = True

	def go_to(self):
		#is dir or file?
		if self.WHERE=='main':
			#it's not a file, it's not a dir! must be... an item in mainmenu
			#stores the 'id' of the section (games, music, etc...) and starts filebrowser
			self.sectionid = self.list.start + self.list.marker
			self.filebrowser()
		elif self.list.isdir():
			#update path in list object and generate a new filelist
			self.list.path = self.list.marked_path()
			try:
				self.list.start, self.list.marker = self.posinlist[self.list.marked_path()]
			except:
				self.list.start = 0
				self.list.marker = 0
			self.list.generate(self.sec_ftypes)
			self.layers[1].empty()
			self.layers[1].add((self.bar))
			self.dolist()
			if len(self.list.data) < 1:
				self.go_back()
		elif self.list.isfile():
			#nothing to do with files at the moment...
			pass

	def filebrowser(self):
		self.WHERE = 'filebrowser'
		self.layers[3].add(self.objectnum)
		self.list.destroy()
		self.layers[0].empty()
		self.ftypes = []
		x = 0
		while 1:
			if x == self.sectionid:
				try:
					self.sec_short = self.cf.val['config.sections.section' + str(x) + '.short']
				except(KeyError):
					break
				self.sec_ftypes = self.cf.val['config.sections.section' + str(x) + '.filetypes']
				self.sec_long = self.cf.val['config.sections.section' + str(x) + '.long']
				self.sec_program = self.cf.val['config.sections.section' + str(x) + '.program']
				self.list.path = self.cf.val['config.sections.section' + str(x) + '.files']
				self.list.main_path = self.cf.val['config.sections.section' + str(x) + '.files']
				break
			x += 1
		if self.sec_program == "QUIT":
			self.quit()
		elif self.sec_ftypes == "EXECUTE":
			self.playlist.pause()
			pygame.display.quit()
			os.system(self.sec_program)
			self.displayinit()
			self.playlist.play()
			self.mainmenu()
		elif os.path.isdir(self.list.path) == False:
			self.mainmenu()
		else:
			self.list.generate(self.sec_ftypes)
			if os.path.isfile(self.themeimg('bg.png')):
				self.bg = load_image(self.themeimg('bg.png'))
			else:
				self.bg = load_image(self.themeimg('filebrowsebg.png'))
			self.screen.blit(self.bg,(0,0))
			self.bar = Image(self.themeimg('filebrowsebar.png'),
					self.th.val['theme.filebrowser.bar.xpos'], self.th.val['theme.filebrowser.bar.ypos'])
			pygame.display.update()
			self.layers[0].add((self.bar))
			self.dolist()
			self.updatepreview()
			if self.sec_program == 'PLAY_MUSIC':
				self.switchsidemenu(True)

	def updatebar(self):
		#y-pos depends on the marker_pos (obviously), the start_pos (bar_y->skin.conf)
		#and the space between the lines (bar_space->skin.conf)
		if self.WHERE == 'main':
			ystart = self.th.val['theme.mainmenu.bar.ypos']
			space = self.th.val['theme.mainmenu.linespace']
		elif self.WHERE == 'sidemenu':
			ystart = self.th.val['theme.sidemenu.ybar']
			space = self.th.val['theme.sidemenu.space']
		else:
			ystart = self.th.val['theme.filebrowser.bar.ypos']
			space = self.th.val['theme.filebrowser.linespace']
		y = self.list.marker * space + ystart
		self.bar.update(self.bar.rect[0], y)
		self.posinlist[self.list.path]=[self.list.start,self.list.marker]

	def go_back(self):
		self.list.start = 0
		self.list.marker = 0
		if self.list.path != '' and self.WHERE == 'filebrowser':
			path = self.list.path
			(self.list.path, spam) = os.path.split(self.list.path)
			if path == self.list.path:
				self.mainmenu()
			#the 'main_path' is always the root-dir for that section
			if len(self.list.path) >= len(self.list.main_path):
				#the 'tmp_*' variables are for going back to the same place in list
				#where it was before
				self.list.start, self.list.marker = self.posinlist[self.list.path]
				#generates a new list
				self.list.generate(self.sec_ftypes)
				self.dolist()
				self.updatebar()
				self.layers[2].empty()
			else:
				#if we reached root, show mainmenu
				self.layers[0].empty()
				self.layers[2].empty()
				self.mainmenu()
			#tmp's are set back, only one 'level' of storing this information
			#is supported, if you go back two dir's, the second will start at begin
			self.list.tmp_start = 0
			self.list.tmp_marker = 0
		elif self.WHERE == 'playlist':
			self.list.restore2()
			self.WHERE = 'filebrowser'
			self.dolist()
			self.updatebar()
		else:
			pass

epMEDIA()
